#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import numpy as np
import math
import os
import matplotlib.pyplot as plt
import time
# Load the mplstyle
#plt.style.use(os.path.join(os.path.split(__file__)[0],
#                           '../', 'bayesvalidrox.mplstyle'))


def convergence_test(engine, steps, error, train_samples, test_samples, test_data, plot = True, write = True, Engine = False):
    """
    Tests the convergence for a metamodel over the number of given training 
    samples in its experimental design.
    The results are written out and plotted.
    
    Conditions: 
        if data is not given, then arbitrary samples will be chosen to run the model on

    Parameters
    ----------
    MetaModel : TYPE
        DESCRIPTION.
        
    model : 
        
    steps : list of ints in increasing order
        The number of samples to train on per iteration
        
    error : function(test_mean, test_std, test_data)
        The function ued to calculate the accuracy of the surrogate, e.g. the L2-norm

    Returns
    -------
    None.

    """
    print('')
    print('STARTING THE CONVERGENCE TEST')
    # Define output lists
    engines = []
    test_runs = {}
    test_runs['mean'] = []
    test_runs['std']=[]
    accuracy = {}
    
    for n in steps:
        print('')
        print(f'---- {n} TRAINING POINTS ----')
        print('')
        samples = train_samples[:n]
        
        # Redo the explerimental design
        engine.ExpDesign.method = 'normal'
        engine.ExpDesign.n_init_samples = n
        engine.ExpDesign.sampling_method = 'user'
        engine.ExpDesign.X = samples
        engine.ExpDesign.Y = None
        engine.start_engine()
        
        # Train and evaluate
        start = time.time()
        engine.train_normal()
        end = time.time()
        
        print('')
        print('Evaluating the metamodel')
        surrogate = engine.MetaModel
        test_mean, test_std = surrogate.eval_metamodel(test_samples)
        
        # Add the results
        engines.append(engine)
        test_runs['mean'].append(test_mean)
        test_runs['std'].append(test_std)
        
        # Add training time
        if 'time' not in accuracy.keys():
            accuracy['time'] = []
        accuracy['time'].append(end-start)
        
        # Add error measures
        er = error(test_mean, test_std, test_data)
        for key in er:
            if key not in accuracy.keys():
                accuracy[key] = []
            accuracy[key].append(er[key])
    
    # Save the results?
    if write:
        None
    
    # Plot the results
    for key in accuracy:
        if plot: 
            plt.plot(steps, accuracy[key], linestyle = 'dashed')
            plt.scatter(steps, accuracy[key], label = key)
            plt.legend()
            plt.show()
        
    return engines, accuracy
